/**********************************************************************
 * Copyright (c) 2003, 2011 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * IBM - Initial API and implementation
 * 
 * $Id: XMLmethodEntryLoader.java,v 1.16 2008-03-12 20:37:17 jkubasta Exp $
 **********************************************************************/
package org.eclipse.hyades.loaders.trace;
import org.eclipse.hyades.loaders.trace.TraceUtils.InvocationInfo;
import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.loaders.util.LookupServiceExtensions;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCTIProfilerCollectionMode;
import org.eclipse.hyades.models.trace.TRCClass;
import org.eclipse.hyades.models.trace.TRCFullMethodInvocation;
import org.eclipse.hyades.models.trace.TRCFullTraceObject;
import org.eclipse.hyades.models.trace.TRCMemoryStatsManager;
import org.eclipse.hyades.models.trace.TRCMethod;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.models.trace.TraceFactory;
import org.eclipse.hyades.models.trace.TracePackage;
import org.eclipse.hyades.models.trace.impl.TRCFullTraceObjectImpl;
import org.eclipse.hyades.models.util.ModelDebugger;
/**
 * @author slavescu
 *  
 */
public class XMLmethodEntryLoader extends TraceMethodBaseLoader {
	
	/** The number of TRCFullMethodInvocations created by this method. */
	private static int _numMethodInvocsCreated = 0;
	
	/** The number of TRCFullMethodInvocations to create, before we update the number
	 * in the stats object of the containing process */
	private static final int NUM_METHOD_INVOCS_BETWEEN_UPDATE = 32768;
	
	//~ Static fields/initializers
	// -----------------------------------------------------------------
	//    private static final String THREAD_ID_REF = "threadIdRef";
	//	protected static final String TICKET = "ticket";
	//	protected static final String STACK_DEPTH = "stackDepth";
	public void addAttribute(String name, String value) {
		if (!loadToModel) {
			return;
		}
		super.addAttribute(name, value);
	}
	
	public void addYourselfInContext() {
		if (!loadToModel) {
			return;
		}
		if(context.isFilterOn())
		{
			if(context.getFilterEngine().isFiltered(TracePackage.eINSTANCE.getTRCMethod_Name(),LoadersUtils.getLookUpKey(methodIdRef)))
			{
				return;
			}
		}
		if(_remoteInvocationContext)
			_activeInvocationContext=true;
		
		theProcess = getProcess();
		theThread = getThreadByIdRef(theProcess);
		
        // Set the TI collection mode and association type if needed		
		TRCAgent a = theProcess.getAgent();
		if(a != null && a.getAgentProxy() != null) {
			TRCAgentProxy ap = a.getAgentProxy();
        	if(ap.getTiCollectionMode() == null || ap.getTiCollectionMode() == TRCTIProfilerCollectionMode.UNKNOWN) {
        		ap.setTiCollectionMode(TRCTIProfilerCollectionMode.EXEC_FLOW);
        		XMLruntimeInitDoneLoader.utilSetAssociation(a.getAgentProxy());
        	}
		}
		
		
		_cs = (CallStackPerThread) LookupServiceExtensions.getInstance().locate(context, CallStackPerThread.class, threadIdRef);
		if (_cs == null) {
			Object csKey = LoadersUtils.getLookUpKey(threadIdRef);
			_cs = new CallStackPerThread();
			LoadersUtils.registerGenericLookUpEntry(context, csKey, _cs);
		}
		_invocationPool = _cs.invocationPool;
		dispatchProcessMode(ProcessSteps.ALL);
		setMaxStackDepth(theThread);
	}
	public void cleanUp(HierarchyContext context) {
		LookupServiceExtensions.getInstance().deregister(context, CallStackPerThread.class);
		super.cleanUp();
	}
	public void endChild(String name) {
	}
	public void initialize(HierarchyContext context, String name) {
		loadToModel = context.isLoadToModel();
		if (!loadToModel) {
			return;
		}
		super.initialize(context, name);
		_fullInvoker = null;
	}
	public void startChild(String name) {
		if (!loadToModel) {
			return;
		}
		if (name.equals(INVOCATION_CONTEXT)) {
			setNullInvocationContext();
			_activeInvocationContext = true;
		} else
			super.startChild(name);
	}

	protected void processENI(int step) {
		super.processEF(step);
		theClass = getClassByIdRef(theThread, TRCFullTraceObjectImpl.class, 0);
		theMethod = getMethodByIdRef(theClass);
		
		/* Create the method invocation element */
		_fullInvocation = TraceFactory.eINSTANCE.createTRCFullMethodInvocation();
		_fullInvocation.setMethodEnterTraceOrder(TraceOrderManager.getInstance().getNextTraceOrderId(theProcess));
		_fullInvocation.setStackDepth(_stackDepth);
		_fullInvocation.setEntryTime(createDeltaTime());
		_fullInvocation.setOrigEntryTime(time);		
		_fullInvocation.setTicket(_ticket);
		_fullInvocation.setMethod(theMethod);
		_fullInvocation.setThread(theThread);
		_fullInvocation.setProcess(theProcess);
		
		_numMethodInvocsCreated++;
		if(_numMethodInvocsCreated % NUM_METHOD_INVOCS_BETWEEN_UPDATE == 0)  {
			updateMethodInvocsCount();
		}
		
        
		addInputOutputValues();
        
		synchronized (_REMOTE_INVOCATION_RESOLUTION_LOCK) {
			setInvokerMethod(theThread, _fullInvocation);
			updateForwardInvokes(_fullInvocation);
		}
		
		if (_cs.isEmpty()) {
			theThread.getInitialInvocations().add(_fullInvocation);
		}
		if(_useUpdateStatisticalInfoOld)
			updateStatisticalInfo(theThread, _fullInvoker, _fullInvocation, theMethod, (TRCFullTraceObject) theObject, theClass);
		else
			updateStatisticalInfoNew(theThread, _fullInvoker, _fullInvocation, theMethod, (TRCFullTraceObject) theObject, theClass);

		_cs.push(_invocationPool.allocInvocation(_fullInvocation, _cs));
		//        LookupServiceExtensions.getInstance().register(context,
		// fullInvocation);
	}
	
	/**
	 *  
	 */
	protected void processEF(int step) {
		super.processEF(step);
		theClass = getClassByIdRef(theThread, TRCFullTraceObjectImpl.class, 0);
		theMethod = getMethodByIdRef(theClass);
		if(createClassObjects) // treat TRCCollectionMode.EXECUTION_NO_INSTANCES case
		{
			if (objIdRef > 0) {
				theObject = getObjectByIdRef(objIdRef, TRCFullTraceObjectImpl.class);
			} else {
				theObject = getClassObject(theMethod.getDefiningClass(), TRCFullTraceObjectImpl.class);
			}
			if (theObject == null) {
				theObject = TraceFactory.eINSTANCE.createTRCFullTraceObject();
			}
		}
		
		/* Create the method invocation element */
		_fullInvocation = TraceFactory.eINSTANCE.createTRCFullMethodInvocation();
		_fullInvocation.setMethodEnterTraceOrder(TraceOrderManager.getInstance().getNextTraceOrderId(theProcess));
		_fullInvocation.setStackDepth(_stackDepth);
		_fullInvocation.setOrigEntryTime(time);
		_fullInvocation.setEntryTime(createDeltaTime());
		_fullInvocation.setTicket(_ticket);
		_fullInvocation.setMethod(theMethod);
		_fullInvocation.setThread(theThread);
		_fullInvocation.setProcess(theProcess);
		_fullInvocation.setOwningObject((TRCFullTraceObject) theObject);
        
		_numMethodInvocsCreated++;
		if(_numMethodInvocsCreated % NUM_METHOD_INVOCS_BETWEEN_UPDATE == 0)  {
			updateMethodInvocsCount();
		}
		
		addInputOutputValues();
        
		synchronized (_REMOTE_INVOCATION_RESOLUTION_LOCK) {
			setInvokerMethod(theThread, _fullInvocation);
			updateForwardInvokes(_fullInvocation);
		}
		
		if (_cs.isEmpty()) {
			theThread.getInitialInvocations().add(_fullInvocation);
		}
		if(_useUpdateStatisticalInfoOld)
			updateStatisticalInfo(theThread, _fullInvoker, _fullInvocation, theMethod, (TRCFullTraceObject) theObject, theClass);
		else
			updateStatisticalInfoNew(theThread, _fullInvoker, _fullInvocation, theMethod, (TRCFullTraceObject) theObject, theClass);

		_cs.push(_invocationPool.allocInvocation(_fullInvocation, _cs));
		//        LookupServiceExtensions.getInstance().register(context,
		// fullInvocation);
	}
	/**
	 *  
	 */
	protected void processES(int step) {
		super.processES(step);
		theClass = getClassByIdRef(theThread, TRCFullTraceObjectImpl.class, 0);
		theMethod = getMethodByIdRef(theClass);
		if(createClassObjects)
		{
			if (objIdRef > 0) {
				virtualObject = (VirtualObjectInfo) LookupServiceExtensions.getInstance().locate(context, VirtualObjectInfo.class, objIdRef);
				if (virtualObject != null) {
					theObject = getClassObject(virtualObject.myClass, TRCFullTraceObjectImpl.class);
				}
			} else {
				theObject = getClassObject(theClass, TRCFullTraceObjectImpl.class);
			}
		}
		updateStatisticalInfoOnly(theMethod, (TRCFullTraceObject) theObject, theClass, theThread, createDeltaTime());
	}

	/** Update the object in our parent that keeps track of how many
		profiling objects exist in the current workbench. */	
	private void updateMethodInvocsCount() {
		TRCMemoryStatsManager m = theProcess.getProcessMemStats();
		if(m == null) {
			m = TraceFactory.eINSTANCE.createTRCMemoryStatsManager();
			theProcess.setProcessMemStats(m);
		}
		m.setNumFullMethInvocObjs(m.getNumFullMethInvocObjs()+NUM_METHOD_INVOCS_BETWEEN_UPDATE);		
	}
	
	protected void updateStatisticalInfoNew(TRCThread aThread, TRCFullMethodInvocation invoker, TRCFullMethodInvocation invocation, TRCMethod aMethod, TRCFullTraceObject anObject, TRCClass aClass) {
		try {
			aMethod.setCalls(aMethod.getCalls() + 1);
			theProcess.setCalls(theProcess.getCalls() + 1);
			theProcess.setLastEventTime(invocation.getEntryTime());
			if (anObject != null) {
				anObject.setCalls(anObject.getCalls() + 1);
				_invokerObjectClass = getExtendedClass(anObject, aClass);
				if (_invokerObjectClass != null) {
					_invokerObjectClass.setInheritedCalls(_invokerObjectClass.getInheritedCalls() + 1);
					_invokerObjectClass.getPackage().setInheritedCalls(_invokerObjectClass.getPackage().getInheritedCalls() + 1);
					_invokerObjectClass.getPackage().getProcess().setInheritedCalls(_invokerObjectClass.getPackage().getProcess().getInheritedCalls() + 1);
				}
			}
			aClass.setCalls(aClass.getCalls() + 1);
			aClass.getPackage().setCalls(aClass.getPackage().getCalls() + 1);
//			if (invoker != null) {
//				int size = invoker.getInvokes().size();
//				if (size > 1) {
//					previousSiblingExitTime = ((TRCFullMethodInvocation) invoker.getInvokes().get(size - 2)).getExitTime();
//					deltaBaseTime = invocation.getEntryTime() - previousSiblingExitTime;
//				} else {
//					//this is first call from "invoker" to "invocation"
//					deltaBaseTime = invocation.getEntryTime() - invoker.getEntryTime();
//				}
//				invokerObject = (TRCFullTraceObject) invoker.getOwningObject();
//				invokerMethod = invoker.getMethod();
//				invokerClass = invoker.getMethod().getDefiningClass();
//				invokerObjectClass = getExtendedClass(invokerObject, invokerClass);
//				//update base and cumulative using deltaBaseTime, invokerObject
//				// and invokerClass
//				updateTimeStatistics();
//			}
			
//			if(fullInvocation.getInvokedBy()!=null)
//			{
//				TRCFullMethodInvocation parent = (TRCFullMethodInvocation)fullInvocation.getInvokedBy();
//				deltaCumulativeTime = fullInvocation.getEntryTime()-parent.getEntryTime();
//				deltaBaseTime = deltaCumulativeTime;
//				updateTimeStatisticsNew();
//			}
		} catch (Exception e) {
			ModelDebugger.log(e);
		}
	}
	
	/**
	 * Method updateStatisticalInfo.
	 * 
	 * @param invocation
	 * @param method
	 * @param theObject
	 * @param theClass
	 */
	protected void updateStatisticalInfo(TRCThread aThread, TRCFullMethodInvocation invoker, TRCFullMethodInvocation invocation, TRCMethod aMethod, TRCFullTraceObject anObject, TRCClass aClass) {
		try {
			aMethod.setCalls(aMethod.getCalls() + 1);
			theProcess.setCalls(theProcess.getCalls() + 1);
			theProcess.setLastEventTime(invocation.getEntryTime());
			if (anObject != null) {
				anObject.setCalls(anObject.getCalls() + 1);
				_invokerObjectClass = getExtendedClass(anObject, aClass);
				if (_invokerObjectClass != null) {
					_invokerObjectClass.setInheritedCalls(_invokerObjectClass.getInheritedCalls() + 1);
					_invokerObjectClass.getPackage().setInheritedCalls(_invokerObjectClass.getPackage().getInheritedCalls() + 1);
					_invokerObjectClass.getPackage().getProcess().setInheritedCalls(_invokerObjectClass.getPackage().getProcess().getInheritedCalls() + 1);
				}
			}
			aClass.setCalls(aClass.getCalls() + 1);
			aClass.getPackage().setCalls(aClass.getPackage().getCalls() + 1);
			if (invoker != null) {
				int size = invoker.getInvokes().size();
				if (size > 1) {
					_previousSiblingExitTime = ((TRCFullMethodInvocation) invoker.getInvokes().get(size - 2)).getExitTime();
					_deltaBaseTime = invocation.getEntryTime() - _previousSiblingExitTime;
				} else {
					//this is first call from "invoker" to "invocation"
					_deltaBaseTime = invocation.getEntryTime() - invoker.getEntryTime();
				}
				_invokerObject = (TRCFullTraceObject) invoker.getOwningObject();
				_invokerMethod = invoker.getMethod();
				_invokerClass = invoker.getMethod().getDefiningClass();
				_invokerObjectClass = getExtendedClass(_invokerObject, _invokerClass);
				//update base and cumulative using deltaBaseTime, invokerObject
				// and invokerClass
				updateTimeStatistics();
			}
		} catch (Exception e) {
			ModelDebugger.log(e);
		}
	}
	/**
	 * Method updateStatisticalInfoOnly.
	 */
	private void updateStatisticalInfoOnly(TRCMethod aMethod, TRCFullTraceObject anObject, TRCClass aClass, TRCThread aThread, double entryTime) {
		InvocationInfo invoker = null;
		if (!_cs.isEmpty()) {
			invoker = (InvocationInfo) _cs.peek();
		}
		updateStatisticalInfoOnlyExtra(invoker, entryTime, aMethod, anObject, aClass);
		_cs.push(_invocationPool.allocInvocation(_cs, anObject, aClass, aMethod, entryTime));
		theProcess.setCalls(theProcess.getCalls() + 1);
		theProcess.setLastEventTime(entryTime);
	}
	private void updateStatisticalInfoOnlyExtra(InvocationInfo invoker, double entryTime, TRCMethod aMethod, TRCFullTraceObject anObject, TRCClass aClass) {
		try {
			aMethod.setCalls(aMethod.getCalls() + 1);
			if (anObject != null) {
				anObject.setCalls(anObject.getCalls() + 1);
				_invokerObjectClass = getExtendedClass(anObject, aClass);
				if (_invokerObjectClass != null) {
					_invokerObjectClass.setInheritedCalls(_invokerObjectClass.getInheritedCalls() + 1);
					_invokerObjectClass.getPackage().setInheritedCalls(_invokerObjectClass.getPackage().getInheritedCalls() + 1);
					_invokerObjectClass.getPackage().getProcess().setInheritedCalls(_invokerObjectClass.getPackage().getProcess().getInheritedCalls() + 1);
				}
			}
			aClass.setCalls(aClass.getCalls() + 1);
			aClass.getPackage().setCalls(aClass.getPackage().getCalls() + 1);
			aClass.getPackage().getProcess().setCalls(aClass.getPackage().getProcess().getCalls() + 1);
			if (invoker != null) {
				_previousSiblingExitTime = invoker.getLastChildExitTime();
				if (_previousSiblingExitTime > 0) {
					_deltaBaseTime = entryTime - _previousSiblingExitTime + invoker.getLastChildOverhead(); // bugzilla 123628
				} else {
					//this is first call from "invoker" to "invocation"
					_deltaBaseTime = entryTime - invoker.getEntryTime();
				}
				_invokerObject = invoker.getObject();
				_invokerClass = invoker.getTheClass();
				_invokerObjectClass = invoker.getObjectClass();
				_invokerMethod = invoker.getMethod();
				//update base and cumulative using deltaBaseTime, invokerObject
				// and invokerClass
				updateTimeStatistics();
			}
		} catch (Exception e) {
			ModelDebugger.log(e);
		}
	}
}