/**********************************************************************
 * Copyright (c) 2003, 2011 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLtraceStartLoader.java,v 1.8 2006-02-08 20:57:10 slavescu Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 * 
 * $Id: XMLtraceStartLoader.java,v 1.8 2006-02-08 20:57:10 slavescu Exp $
 **********************************************************************/
package org.eclipse.hyades.loaders.trace;

import java.math.BigDecimal;

import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.loaders.util.LookupServiceExtensions;
import org.eclipse.hyades.models.hierarchy.HierarchyPackage;
import org.eclipse.hyades.models.trace.TRCProcess;


/**
 * @author slavescu
 *
 */
public class XMLtraceStartLoader extends TraceXMLFragmentLoader {
    //~ Instance fields ----------------------------------------------------------------------------
	
    protected static final String TRACE_ID = "traceId";
    protected static final String CPU_SAMPLE_RESOLUTION = "cpuSampleResolution";
    protected String traceId;
    protected double cpuSampleResolution;

    protected BigDecimal absoluteTime;
    //~ Methods ------------------------------------------------------------------------------------

    public void addAttribute(String name, String value) {
    	
        switch (LoadersUtils.getHashCode(name)) {
            case TraceConstants.TRACE_ID_int:
                traceId = value;

                break;
            case TraceConstants.CPU_SAMPLE_RESOLUTION_int:
            	cpuSampleResolution = Double.parseDouble(value);

                break;
                
            case TraceConstants.TIME_int:
            	absoluteTime = new BigDecimal(value.trim());
            	// Flow through to default so that parent can parse time, as well
            	
            default:
                super.addAttribute(name, value);

                break;
            }
    }

    public void addYourselfInContext() {
    	
        //        zeroTime = time;
		if(context.getAgent()!=null && !context.getAgent().eIsSet(HierarchyPackage.eINSTANCE.getTRCAgent_StartTime()))
		{
			context.getAgent().setStartTime(time);
			if(context.getAgent().getAgentProxy()!=null)
				context.getAgent().getAgentProxy().setStartTime(time);
		}

        TRCProcess p = getProcess();

        if (p.getStartTime() == 0) {
            p.setStartTime(createDeltaTime());
        }
        p.setId(traceId);
        if(cpuSampleResolution!=0)
        	p.setCpuSampleResolution(cpuSampleResolution);
        if(precision!=0)
        	p.setPrecision(precision);
        if(language==null || language.trim().length()==0)
        	language="Java"; //make the default Java if language attribute is null or empty
       	addLanguageIfRequired(p.getLanguages());
       	
       	addAnnotationsIfRequired(p);
       	
    	if(absoluteTime != null) {
    		p.setAbsoluteTime(absoluteTime);
    	}
    	       	
        // cleanup the call stack for previous trace (if exists)
        LookupServiceExtensions.getInstance().deregister(context, CallStackPerThread.class);
        
    }

    public void initialize(HierarchyContext context, String name) {
        super.initialize(context, name);
        traceId = null;
        cpuSampleResolution=0;
        precision=0;
        annotations.clear();
    }
}
