/**********************************************************************
 * Copyright (c) 2005, 2012 IBM Corporation, Intel Corporation.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: IProfileLaunchConfigurationConstants.java,v 1.7 2007-05-19 00:50:58 jkubasta Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.trace.ui.internal.launcher;

import org.eclipse.hyades.trace.ui.*;

/**
 * @author znensi
 *
 * Constants for storing profiling options/parameters in an <code>
 * ILaunchConfiguration</code>. All the data must be converted to string
 * form in order to persist after the workbench is shut down (no object
 * references allowed).
 * 
 * @see ILaunchConfiguration#getAttribute()
 * @see ILaunchConfiguration#setAttribute()
 */
public interface IProfileLaunchConfigurationConstants
{
	/**
	 * A <code>String</code> containing the hostname, e.g. "localhost".
	 */
	public static final String ATTR_HOSTNAME = UIPlugin.getPluginId() + ".ATTR_HOSTNAME";

	/**
	 * An <code>int</code> containing the port number, e.g. 10002.
	 */
	public static final String ATTR_PORT = UIPlugin.getPluginId() + ".ATTR_PORT";

	/**
	 * A <code>String</code> representing the selected agents in the form of:
	 * "pid1:agentName1:dataCollectorId1:analysisTypeID1,analysisTypeID2;pid2:agentName1:dataCollectorId1:analysisTypeID1..."
	 */
	public static final String ATTR_AGENTS = UIPlugin.getPluginId() + ".ATTR_AGENTS0";

	/**
	 * A <code>String</code> containing the classpath as it would appear on
	 * the command line, e.g. ".:/dir:/dir/file.jar"
	 */
	public static final String ATTR_CLASSPATH = UIPlugin.getPluginId() + ".ATTR_CLASSPATH";

	/**
	 * A <code>String</code> containing the name of the project in which to
	 * store the profiling data, e.g. "ProfileProject".
	 */
	public static final String ATTR_DESTINATION_PROJECT = UIPlugin.getPluginId() + ".ATTR_DESTINATION_PROJECT";
	
	/**
	 * A <code>String</code> containing the name of the monitor to profile
	 * under, e.g. "DefaultMonitor".
	 */
	public static final String ATTR_DESTINATION_MONITOR = UIPlugin.getPluginId() + ".ATTR_DESTINATION_MONITOR";

	/**
	 * A <code>String</code> containing the name of the file to
	 * send profiling data to.
	 */
	public static final String ATTR_DESTINATION_FILE = UIPlugin.getPluginId() + ".ATTR_DESTINATION_FILE";

	/**
	 * A <code>boolean</code> specifying whether or not the
	 * profiling data should be written to a file.
	 */
	public static final String ATTR_PROFILE_TO_FILE = UIPlugin.getPluginId() + ".ATTR_PROFILE_TO_FILE";
	
	/**
	 * A <code>String</code> specifying the id of the active profiling
	 * set.
	 */
	public static final String ATTR_PROFILING_SET = UIPlugin.getPluginId() + ".ATTR_PROFILING_SET";

	/**
	 * A <code>boolean</code> specifying whether or not the active filter set
	 * should be configured automatically
	 */
	public static final String ATTR_AUTO_FILTER_CRITERIA = UIPlugin.getPluginId() + ".ATTR_AUTO_FILTER_CRITERIA";

	/**
	 * A <code>String</code> specifying the id of the active filter
	 * set.
	 */
	public static final String ATTR_FILTER_SET = UIPlugin.getPluginId() + ".ATTR_FILTER_SET";

	/**
	 * A <code>boolean</code> specifying whether or not instance-level
	 * information should be collected for Heap.
	 */
	public static final String ATTR_SHOW_HEAP_INSTANCES = UIPlugin.getPluginId() + ".ATTR_SHOW_HEAP_INSTANCES";

	/**
	 * A <code>boolean</code> specifying whether or not instance-level
	 * information should be collected for Execition Flow.
	 */
	public static final String ATTR_SHOW_EXEC_INSTANCES = UIPlugin.getPluginId() + ".ATTR_SHOW_EXEC_INSTANCES";

	
	/**
	 * A <code>boolean</code> specifying <code>true</code> if the "Show
	 * Execution Statistics Details" box is checked, or <code>false</code> if
	 * the "Show Execution Flow" box is checked.
	 */
	public static final String ATTR_SHOW_EXECUTION_FLOW = UIPlugin.getPluginId() + ".ATTR_SHOW_EXECUTION_FLOW";
	
	/**
	 * A <code>boolean</code> indicating whether method CPU time information should
	 * be collected. 
	 */
	public static final String ATTR_SHOW_EXEC_CPU_TIME = UIPlugin.getPluginId() + ".ATTR_SHOW_EXEC_CPU_TIME";

	/**
	 * A <code>boolean</code> indicating whether boundary classes should be
	 * collected.
	 */
	public static final String ATTR_SHOW_EXEC_BOUNDARY_CLASS = UIPlugin.getPluginId() + ".ATTR_SHOW_EXEC_BOUNDARY_CLASS";
	
	/**
	 * An <code>int</code> indicating the boundary class depth.  <code>ATTR_SHOW_EXEC_BOUNDARY_CLASS</code> should
	 * be <code>true</code> for this attribute to have any meaning. 
	 */
	public static final String ATTR_SHOW_EXEC_BOUNDARY_CLASS_DEPTH = UIPlugin.getPluginId() + ".ATTR_SHOW_EXEC_BOUNDARY_CLASS_DEPTH";
	
	/**
	 * A <code>boolean</code> indicating whether the polling frequency mode is automatic or manually set to a constant.
	 * The flag is expected to be true if its automatically calculated or false if it's manually set.
	 */
	public static final String ATTR_EXEC_POLLING_FREQ_MODE = UIPlugin.getPluginId() + ".ATTR_EXEC_POLLING_FREQ_MODE";

	/**
	 * An <code>int</code> indicating the polling frequency period
	 */
	public static final String ATTR_EXEC_POLLING_FREQ = UIPlugin.getPluginId() + ".ATTR_EXEC_POLLING_FREQ";
	
	
	/**
	 * A <code>boolean</code> specifying whether or not the number of method
	 * invocations should be limited.
	 */
	public static final String ATTR_PROFILING_LIMIT_INVOCATIONS = UIPlugin.getPluginId() + ".ATTR_PROFILING_LIMIT_INVOCATIONS";

	/**
	 * An <code>int</code> containing the number of method invocations to
	 * limit to (-1 if no limit).
	 */
	public static final String ATTR_PROFILING_LIMIT_INVOCATIONS_NUM = UIPlugin.getPluginId() + ".ATTR_PROFILING_LIMIT_INVOCATIONS_NUM";

	/**
	 * A <code>boolean</code> specifying whether or not the number of seconds
	 * to limit profiling to is selected.
	 */
	public static final String ATTR_PROFILING_LIMIT_SECONDS = UIPlugin.getPluginId() + ".ATTR_PROFILING_LIMIT_SECONDS";

	/**
	 * An <code>int</code> containing the number of seconds to profile for
	 * (-1 if no limit).
	 */
	public static final String ATTR_PROFILING_LIMIT_SECONDS_NUM = UIPlugin.getPluginId() + ".ATTR_PROFILING_LIMIT_SECONDS_NUM";
	
	/**
	 * A <code>boolean</code> specifying whether or not the agent should
	 * automatically start monitoring upon launch/attach.
	 */
	public static final String ATTR_AUTO_MONITORING = UIPlugin.getPluginId() + ".ATTR_AUTO_MONITORING";
	
	/**
	 * A <code>String</code> indicating the data collector mechanisms along with the analysis types that
	 * are selected.  The selection is serialized as a string using the follwoing format:
	 * <p>
	 * DATA_COLLECTOR_ID_1:ANALYSIS_TYPE_ID_1,ANALYSIS_TYPE_ID_2;DATA_COLLECTOR_ID_2:ANALYSIS_TYPE_ID1,ANALYSIS_TYPE_ID2 and so on ...
	 * The analysis type id can also be a profiling type id (to support backward compatibility)
	 * </p>
	 */
	public static final String ATTR_PROFILING_COLLECTOR_AND_ANALYSIS = UIPlugin.getPluginId() + ".ATTR_DATA_COLLECTOR";
	
	/**
	 * A <code>String</code> indicating the path to the Java executable that should be used for the 
	 * launch. Defaults to "java.exe", which references the default application alias configured
	 * within the AC.
	 */
	public static final String ATTR_JAVA_EXE = UIPlugin.getPluginId() + ".ATTR_JAVA_EXE";
	public static final String ATTR_JVM_ID = UIPlugin.getPluginId() + ".ATTR_JVM_ID";
	
	public final static String LIB_PREFIX = System.getProperty("os.name").toLowerCase().indexOf("windows") < 0 ? "lib" : "";
	public final static String LIB_SUFFIX = System.getProperty("os.name").toLowerCase().indexOf("windows") < 0 ? ".so" : ".dll";

	public final static boolean NOVERIFY_JAVA7 = Boolean.getBoolean( "org.eclipse.tptp.noVerifyJava7" );
	
	public static final String ATTR_PROFILING_GENERIC_AGENT_ATTACH = UIPlugin.getPluginId() + ".ATTR_PROFILING_GENERIC_AGENT_ATTACH";
	
}

