/**********************************************************************
 * Copyright (c) 2005, 2014 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: TIUtils.java,v 1.1 2005-07-22 14:49:38 hleung Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.uml2sd.trace.util;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.hyades.loaders.trace.TraceUtils;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.hierarchy.TRCAgentProxy;
import org.eclipse.hyades.models.hierarchy.TRCCollectionMode;
import org.eclipse.hyades.models.hierarchy.TRCConfiguration;
import org.eclipse.hyades.models.hierarchy.TRCMonitor;
import org.eclipse.hyades.models.hierarchy.TRCNode;
import org.eclipse.hyades.models.hierarchy.TRCOption;
import org.eclipse.hyades.models.hierarchy.TRCProcessProxy;
import org.eclipse.hyades.models.trace.TRCFullMethodInvocation;
import org.eclipse.hyades.models.trace.TRCMethodInvocation;
import org.eclipse.hyades.models.trace.TRCProcess;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.trace.internal.ui.PDContentProvider;
import org.eclipse.hyades.trace.ui.HyadesConstants;
import org.eclipse.hyades.trace.ui.HyadesUtil;
import org.eclipse.hyades.ui.extension.INavigatorItem;
import org.eclipse.hyades.ui.internal.navigator.INavigator;
import org.eclipse.hyades.uml2sd.trace.TraceSDUtil;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceProcess;
import org.eclipse.hyades.uml2sd.trace.loaders.internal.TraceProcesses;
import org.eclipse.hyades.uml2sd.ui.actions.widgets.Criteria;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

/**
 * Utils
 */
public class TIUtils {
	
	protected static int _treeLevel;
	protected static String _tableText;
	protected static int[] _maxValueLen;
	protected static int _numCols;
	protected static final int COLUMN_SPACING = 4;
	
	protected enum ActionFlag {
		READ_ROW, COLUMN_ALIGNMENT;
	}
	
	/**
	 * This method searches for the reference of the TRCProcess 
	 * selected in the profiling monitor view
	 */
	static public void searchTRCProcesses(EObject object, TraceProcesses list, IProgressMonitor monitor) {
		
		if (monitor.isCanceled()) {
			return;
		}
		if (object instanceof TRCMonitor) {
			TRCMonitor m = (TRCMonitor)object;
			if (m.getNodes() == null ||
				m.getNodes().isEmpty()) {
				return;
			}
			for (Iterator<TRCNode> i = m.getNodes().iterator(); i.hasNext() && !monitor.isCanceled(); ) {
				searchTRCProcesses((EObject)i.next(), list, monitor);
			}
		} else if (object instanceof TRCNode) {
			TRCNode n = (TRCNode)object;
			if (n.getProcessProxies() == null ||
				n.getProcessProxies().isEmpty()) {
				return;
			}
			for (Iterator<TRCProcessProxy> i = n.getProcessProxies().iterator(); i.hasNext() && !monitor.isCanceled(); ) {
				searchTRCProcesses((EObject)i.next(), list, monitor);
			}
		} else if (object instanceof TRCProcessProxy) {
			TRCProcessProxy pp = (TRCProcessProxy)object;
			if (pp.getAgentProxies() == null ||
				pp.getAgentProxies().isEmpty()) {
				return;
			}
			for (Iterator<TRCAgentProxy> i = pp.getAgentProxies().iterator(); i.hasNext() && !monitor.isCanceled(); ) {
				searchTRCProcesses((EObject)i.next(), list, monitor);
			}
		} else if (object instanceof TRCAgentProxy &&
				   ((TRCAgentProxy) object).getType().equals(HyadesConstants.PROFILE_AGENT_TYPE)) {
			TRCAgent agent = ((TRCAgentProxy) object).getAgent();
			TRCProcess process = TraceUtils.getProcess(agent);
			if (process != null) {
				EList<TRCMethodInvocation> invocations = process.getInvocations();
				if (invocations != null &&
					invocations.size() > 0 &&
					invocations.get(0) instanceof TRCFullMethodInvocation) {
					list.add(new TraceProcess(process));
				}
			}
		}
	}

	public static List<TRCAgentProxy> getProfileAgents(TRCProcessProxy processProxy) {
		List<TRCAgentProxy> profileAgents = new ArrayList<TRCAgentProxy>();
		if(processProxy!=null){
			EList<TRCAgentProxy> agents = processProxy.getAgentProxies();
			TRCAgentProxy agentProxy = null;
			for (Iterator<TRCAgentProxy> iterator = agents.iterator();	iterator.hasNext();	) {
				agentProxy = iterator.next();
				if(HyadesConstants.PROFILE_AGENT_TYPE.equals(agentProxy.getType())) {
					profileAgents.add(agentProxy);
				}					
			}
		}			
		return profileAgents;
	}

	/**
	 * @param methodInvocation
	 * @return
	 */
	public static double getAbsoluteEntryTime(TRCFullMethodInvocation methodInvocation) {
		return getAbsoluteEntryTime(methodInvocation.getProcess().getAgent())+
			   methodInvocation.getEntryTime();
	}

	/**
	 * @param agent
	 * @return
	 */
	public static double getAbsoluteEntryTime(TRCAgent agent) {
		double delta = agent.getAgentProxy().getProcessProxy().getNode().getDeltaTime()*1E-6;
		return delta+agent.getStartTime();
	}

	/**
	 * @param process
	 * @return
	 */
	public static double getAbsoluteEntryTime(TRCProcess process) {
		return getAbsoluteEntryTime(process.getAgent())+
			   process.getStartTime();
	}

	/**
	 * @param thread
	 * @return
	 */
	public static double getAbsoluteEntryTime(TRCThread thread) {
		return getAbsoluteEntryTime(thread.getProcess().getAgent())+
			   thread.getStartTime();
	}

	/**
	 * @param methodInvocation
	 * @return
	 */
	public static double getAbsoluteExitTime(TRCFullMethodInvocation methodInvocation) {
		double start = getAbsoluteEntryTime(methodInvocation.getProcess().getAgent());
		if (methodInvocation.getExitTime() != 0) {
			return start+methodInvocation.getExitTime();
		}
		if (methodInvocation.getInvokes() == null || methodInvocation.getInvokes().size() == 0) {
			return getAbsoluteExitTime(methodInvocation.getThread());
		}
		TRCFullMethodInvocation lastCalled =
			(TRCFullMethodInvocation)methodInvocation.getInvokes().get(methodInvocation.getInvokes().size()-1);
		if (lastCalled.getExitTime() == 0) {
			return getAbsoluteExitTime(lastCalled);
		}
		return start+lastCalled.getExitTime();
	}

	/**
	 * @param agent
	 * @return
	 */
	public static double getAbsoluteExitTime(TRCAgent agent) {
		double delta = agent.getAgentProxy().getProcessProxy().getNode().getDeltaTime()*1E-6;
		return delta+agent.getStopTime();
	}

	/**
	 * @param process
	 * @return
	 */
	public static double getAbsoluteExitTime(TRCProcess process) {
		if (process.getStopTime() == 0) {
			return getAbsoluteExitTime(process.getAgent());
		}
		return getAbsoluteEntryTime(process.getAgent())+
			   process.getStopTime();
	}

	/**
	 * @param thread
	 * @return
	 */
	public static double getAbsoluteExitTime(TRCThread thread) {
		if (thread.getStopTime() == 0) {
			return getAbsoluteExitTime(thread.getProcess().getAgent());
		}
		return getAbsoluteEntryTime(thread.getProcess().getAgent())+
			   thread.getStopTime();
	}
	
	@SuppressWarnings("rawtypes")
	public static boolean doesAgentProxyStillExists(String agentProxyRuntimeId) {
		List monitors = PDContentProvider.getMonitors();
		for (Iterator i = monitors.iterator(); i.hasNext(); ) {
			TRCMonitor monitor = (TRCMonitor)i.next();
			EList<TRCNode> nodes = monitor.getNodes();
			for (Iterator<TRCNode> j = nodes.iterator(); j.hasNext(); ) {
				TRCNode node = j.next();
				EList<TRCProcessProxy> pps = node.getProcessProxies();
				for (Iterator<TRCProcessProxy> k = pps.iterator(); k.hasNext(); ) {
					TRCProcessProxy pp = k.next();
					EList<TRCAgentProxy> aps = pp.getAgentProxies();
					for (Iterator<TRCAgentProxy> l = aps.iterator(); l.hasNext(); ) {
						TRCAgentProxy ap = l.next();
						if (agentProxyRuntimeId.equals(ap.getRuntimeId())) {
							return true;
						}
					}
				}
			}
		}
		return false;
	}

	public static boolean matchCharacter(char c1, char c2, boolean isCaseSensitive) {
		if (TraceSDUtil.debugMatch && unitTestDebug) {
			TraceSDUtil.debugUml2SDTraceMatch("Comparing "+c1+" to "+c2); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return (c1 == c2) ||
			    (!isCaseSensitive &&	Character.toLowerCase(c1) == Character.toLowerCase(c2));
	}
	
	private static boolean trailingStars(char s[], int l) {
		int i;
		for (i = l; i < s.length && s[i] == '*'; i++);
		return i == s.length;
	}
	
	private static final boolean unitTestDebug = false;
			
	public static boolean matchCriteria(String s, Criteria criteria) {
		try {
			String e = criteria.getExpression();
			char ea[] = e.toCharArray();
			if (TraceSDUtil.debugMatch && unitTestDebug) {
				TraceSDUtil.debugUml2SDTraceMatch("Comparing "+s+" to "+e); //$NON-NLS-1$ //$NON-NLS-2$
			}
			int i, j;
			for (i = 0, j = 0; i < s.length() && j < ea.length; i++) {
				char sc = s.charAt(i), ec = ea[j];
				if (TraceSDUtil.debugMatch && unitTestDebug) {
					TraceSDUtil.debugUml2SDTraceMatch("s["+i+"]="+sc+" e["+j+"]="+ec); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
				}
				if (ec == '*') {
					if (j+1 == ea.length) {
						// trailing * -> everything matches
						if (TraceSDUtil.debugMatch && unitTestDebug) {
							TraceSDUtil.debugUml2SDTraceMatch("#1a true"); //$NON-NLS-1$
						}
						return true;
					}
					if (ea[j+1] == '?') {
						 // swap * and ?, it's easier this way...
						ec = ea[j] = '?';
						ea[j+1] = '*';
					}
				}
				if (ec == '*') {
					int k;
					for (k = 1; i+k-1 < s.length() && j+k < ea.length; k++) {
						if (ea[j+k] == '*' || ea[j+k] == '?') {
							j += k;
							i += k - 2;
							break; // another wildcard -> next loop
						}
						if (!matchCharacter(s.charAt(i+k-1), ea[j+k], criteria.isCaseSenstiveSelected())) {
							break; // i will be incremented at the beginning of the next loop
						}
					}
					if (TraceSDUtil.debugMatch && unitTestDebug) {
						TraceSDUtil.debugUml2SDTraceMatch("After '*': s["+i+"]="+sc+" e["+j+"]="+ec); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
					}
					if (i+k-1 == s.length() && trailingStars(ea, j+k)) {
						if (TraceSDUtil.debugMatch && unitTestDebug) {
							TraceSDUtil.debugUml2SDTraceMatch("#1b true"); //$NON-NLS-1$
						}
						return true;
					}
				} else if (ec == '?') {
					if (i+1 < s.length() && j+1 < ea.length) {
						j++;
					} else {
						// trailing ? -> one char matches
						if (TraceSDUtil.debugMatch && unitTestDebug) {
							TraceSDUtil.debugUml2SDTraceMatch(i+1 == s.length() && trailingStars(ea, j+1) ?"#3 true":"#3 false"); //$NON-NLS-1$ //$NON-NLS-2$
						}
						return i+1 == s.length() && trailingStars(ea, j+1);
					}
				} else if (!matchCharacter(sc, ec, criteria.isCaseSenstiveSelected())) {
					if (TraceSDUtil.debugMatch && unitTestDebug) {
						TraceSDUtil.debugUml2SDTraceMatch("#4 false"); //$NON-NLS-1$
					}
					return false;
				} else {
					j++;
				}
			}
			if (TraceSDUtil.debugMatch && unitTestDebug) {
				TraceSDUtil.debugUml2SDTraceMatch(i == s.length() && (j == ea.length || (j+1 == ea.length && (ea[j] == '*' || ea[j] == '?')))?"#5 true":"#5 false"); //$NON-NLS-1$ //$NON-NLS-2$
			}
			return i == s.length() && trailingStars(ea, j);
		} catch (Exception e) {
			if (TraceSDUtil.debugMatch) {
				e.printStackTrace();
			}
			return false;
		}
	}

	/**
	   * @return true if collection mode used to profile this agent is ready for a
	   * UML2 Object Interactions diagram ... or something like this. 
	   */
	  public static boolean hasObjectInteractions( TRCAgentProxy agent_proxy )
	  {    
	    boolean collects_object_instances = false;
	    
	    switch( agent_proxy.getCollectionMode().getValue() )
	    {
	      case TRCCollectionMode.EXECUTION_FULL :
	      case TRCCollectionMode.HEAP_AND_EXECUTION_FULL :
	      case TRCCollectionMode.EXECUTION_NO_INSTANCES :
	        collects_object_instances = true;
	        break;
	        
	      case TRCCollectionMode.HEAP_FULL :
	      case TRCCollectionMode.HEAP_STATISTICS_ONLY :
	      case TRCCollectionMode.HEAP_AND_EXECUTION_STATISTICS_ONLY :
	      case TRCCollectionMode.EXECUTION_STATISTICS_ONLY :
	      case TRCCollectionMode.HEAP_FULL_AND_EXECUTION_STATISTICS_ONLY :
	      default:
	        collects_object_instances = false;
	        break;	    	
	    }
	    
	    return collects_object_instances;
	  }

	/**
		 *
		 */
		public static boolean hasInvocationData(TRCAgentProxy agentProxy) {
			boolean stackInformation = false;
			//boolean monitorMode = false;
			if (agentProxy != null &&
				agentProxy.getType().equals(HyadesConstants.PROFILE_AGENT_TYPE)) {
				EList<TRCConfiguration> configs = agentProxy.getConfigurations();
				if (configs == null) {
					return false;
				}
				for (Iterator<TRCConfiguration> i = configs.iterator(); i.hasNext(); ) {
					TRCConfiguration config = i.next();
					EList<TRCOption> options = config.getOptions();
					if (options == null) {
						continue;
					}
					for (Iterator<TRCOption> io = options.iterator(); io.hasNext(); ) {
						TRCOption option = io.next();
						if (option.getKey().equals("STACK_INFORMATION") && //$NON-NLS-1$
							!option.getValue().equals("none")) { //$NON-NLS-1$
							stackInformation = true;
	//					} else if (option.getKey().equals("MONITOR_MODE") && //$NON-NLS-1$
	//							   option.getValue().equals("all")) { //$NON-NLS-1$
	//						monitorMode = true;
						}
					}
				}
			}
			return /*hasObjectInteractions(agentProxy) &&*/ stackInformation;// && monitorMode;
		}

	/**
	 * This method searches for the reference of the TRCProcess 
	 * selected in the profiling monitor view
	 */
	static public void searchTRCAgentProxies(EObject object, List<TRCAgentProxy> list) {
		
		if (object instanceof TRCMonitor) {
			TRCMonitor m = (TRCMonitor)object;
			if (m.getNodes() == null ||
				m.getNodes().isEmpty()) {
				return;
			}
			for (Iterator<TRCNode> i = m.getNodes().iterator(); i.hasNext(); ) {
				searchTRCAgentProxies(i.next(), list);
			}
		} else if (object instanceof TRCNode) {
			TRCNode n = (TRCNode)object;
			if (n.getProcessProxies() == null ||
				n.getProcessProxies().isEmpty()) {
				return;
			}
			for (Iterator<TRCProcessProxy> i = n.getProcessProxies().iterator(); i.hasNext(); ) {
				searchTRCAgentProxies(i.next(), list);
			}
		} else if (object instanceof TRCProcessProxy) {
			TRCProcessProxy pp = (TRCProcessProxy)object;
			if (pp.getAgentProxies() == null ||
				pp.getAgentProxies().isEmpty()) {
				return;
			}
			for (Iterator<TRCAgentProxy> i = pp.getAgentProxies().iterator(); i.hasNext(); ) {
				searchTRCAgentProxies(i.next(), list);
			}
		} else if (object instanceof TRCAgentProxy) {
			TRCAgentProxy agentProxy = (TRCAgentProxy)object;
			if (/*hasInvocationData(agentProxy) &&*/ !list.contains(agentProxy)) {
				list.add(agentProxy);
			}
		} else if (TraceSDUtil.debugEvents) {
			TraceSDUtil.debugUml2SDTraceEvents("WARNING! searchTRCAgentProxies(): Found a "+object); //$NON-NLS-1$
		}
	}

	/**
	 * This method looks into the selection of the active navigator, usually PDProjectExplorer...
	 * Every selected TRCAgentProxy (or INavigatorItem under it) is collected in the list parameter.
	 * Selection of a TRCMonitor, TRCNode or TRCProcessProxy implies collection of all the
	 * "Profiling" agents under it.
	 * @return
	 */
	@SuppressWarnings("rawtypes")
	public static List<TRCAgentProxy> collectMultiSelectionInNavigator() {
		INavigator nav = HyadesUtil.getActiveNavigator();
	    IStructuredSelection selection = nav == null ? null : nav.getStructuredSelection();
	    List<TRCAgentProxy> list = new ArrayList<TRCAgentProxy>();
	    if (selection != null) {
		    for (Iterator i = selection.iterator(); i.hasNext(); ) {
		    	Object obj = i.next();
	            if( obj instanceof INavigatorItem ) obj = ((INavigatorItem)obj).getData();
	            if (obj instanceof EObject) {
	            	searchTRCAgentProxies((EObject)obj, list);
	            } else if (TraceSDUtil.debugEvents) {
					TraceSDUtil.debugUml2SDTraceEvents("WARNING! collectMultiSelectionInNavigator(): Found a "+obj.getClass().getName()+" "+obj); // TODO debug //$NON-NLS-1$ //$NON-NLS-2$
	            }
		    }
	    }
		return list;
	}

//	private static void testIt(String s, String e, boolean c, boolean ev) {
//		Criteria cr = new Criteria();
//		cr.setCaseSenstiveSelected(c); cr.setExpression(e);
//		boolean res = matchCriteria(s, cr);
//		if (res == ev) {
//			System.out.println("OK : "+s+" "+e+" ["+c+"]="+res+" (expecting "+ev+")");
//		} else {
//			System.out.println(" KO: "+s+" "+e+" ["+c+"]="+res+" (expecting "+ev+")");
//		}
//	}
//
//	public static void main(String args[]) {
//		TraceSDPlugin.debugMatch = true;
//		testIt("ThreadRun:1234", "Th*Run", false, false);
//		testIt("ThreadRun:1234", "Th*Run:123?", false, true);
//		testIt("ThreadRun:1234", "Th*Run?123?", false, true);
//		testIt("ThreadRun:1234", "Th*Run:1?3??", false, false);
//		testIt("ThreadRun:1234", "Th*Run:1?3?*", false, true);
//		testIt("ThreadRun", "Th*Run", true, true);
//		testIt("ThreadRun", "Th*Run", false, true);
//		testIt("ThreadRun", "ThreadRun*****************", true, true);
//		testIt("ThreadRun", "ThreadRu*********?*******", true, true);
//		testIt("ThreadRun", "ThreadRun*********?*******", true, false);
//		testIt("ThrennnadRun", "*n", false, true);
//		testIt("ThrennnadRun", "*N", true, false);
//		testIt("", "*", true, true);
//		testIt("TTT", "*", true, true);
//		testIt("TxxxT", "T???T", true, true);
//		testIt("TxxxT", "T?*?T", true, true);
//		testIt("TxxxT", "T??T", true, false);
//		testIt("T", "?", true, true);
//		testIt("TT", "*T", true, true);
//		testIt("TT", "?T", true, true);
//		testIt("TT", "T*", true, true);
//		testIt("TT", "T?", true, true);
//		testIt("TT", "**", true, true);
//		testIt("TT", "??", true, true);
//		testIt("GetChannels", "*Server*", false, false);
//		testIt("UmtsServer", "*Server", true, true);
//		testIt("UmtsServer", "*Server*", true, true);
//		testIt("UmtsServer", "*server", false, true);
//		testIt("UmtsServer", "*server*", false, true);
//	}
	
	
	/**
	 * Copy a specific row from the TreeViewer object
	 */
	public static void copyCurrentRowContents(Tree tree) {
		_numCols = tree.getColumnCount();
		_tableText = "";
		_treeLevel=0;
		
		// This is used for alignment purposes to keep track of the longest string in each column
		_maxValueLen = new int[_numCols];		
		
		TreeItem[] row = tree.getSelection();
		
		//find the length of string in each column - for alignment purposes
		if (row.length != 0) {
			readRow(row[0], ActionFlag.COLUMN_ALIGNMENT);
		}
				
		//Copy tree header to a string
		for(int i=0; i<_numCols; i++) {
			int length = tree.getColumn(i).getText().length();
			
			//find the length of longest string in each column - for alignment purposes
			_maxValueLen[i] = (_maxValueLen[i] > length) ? _maxValueLen[i] : length;
			
			//Now we have the longest length for each column. We add some characters to create some space between columns
			_maxValueLen[i] += COLUMN_SPACING;
			
			//Amount of spacing to be done after the word to align the next column data
			length = _maxValueLen[i] - length;
			
			//Add the word as well as the number of spaces
			_tableText += tree.getColumn(i).getText();
			_tableText += repeat(" ", length);
		}
		
		_tableText += "\n";
		_treeLevel=0;
		
		//Copy the table data to the string. The required number of spaces are appended to each data entry for alignment of columns
		if (row.length != 0) {
			_tableText += readRow(row[0], ActionFlag.READ_ROW);
		}
		
		// transfer to system's clip board
		Clipboard clipboard = new Clipboard(Display.getCurrent());
	    TextTransfer textTransfer = TextTransfer.getInstance();
	    Object[] data = new Object[]{_tableText};
	    Transfer[] transfers = new Transfer[]{textTransfer};
	    clipboard.setContents(data, transfers);
	    clipboard.dispose();
	}
	
	
	/**
	 * Copy all the visible (or expanded) rows within the TreeViewer object
	 */
	public static void copyTreeContents(Tree tree) {
		_numCols = tree.getColumnCount();
		
		_tableText = "";
		_treeLevel = 0;
		_maxValueLen = new int[_numCols];
		
		TreeItem[] treeItems = tree.getItems();
		for (int i=0; i<treeItems.length; i++) {
			readChildrenData(treeItems[i], ActionFlag.COLUMN_ALIGNMENT);
		}
				
		//Copy tree header to string
		for(int i=0; i<_numCols; i++) {
			int length = tree.getColumn(i).getText().length();
			_maxValueLen[i] = (_maxValueLen[i] > length) ? _maxValueLen[i] : length;
			
			//At this point, we have the length of the longest string for each column. We add some characters to create some space between columns
			_maxValueLen[i] += COLUMN_SPACING;
			
			//We find the amount of spaces to be left after the word to align the next column data
			length = _maxValueLen[i] - length;
			_tableText += tree.getColumn(i).getText();
			_tableText += repeat(" ", length);
		}
		_tableText += "\n";
		
		_treeLevel = 0;
		//Copy the table data to the string. The required number of spaces are appended to each data entry for alignment of columns
		for (int i=0; i<treeItems.length; i++) {
			readChildrenData(treeItems[i], ActionFlag.READ_ROW);
		}
		
		// transfer to system's clip board
		Clipboard clipboard = new Clipboard(Display.getCurrent());
	    TextTransfer textTransfer = TextTransfer.getInstance();
	    Object[] data = new Object[]{_tableText};
	    Transfer[] transfers = new Transfer[]{textTransfer};
	    clipboard.setContents(data, transfers);
	    clipboard.dispose();
	}
	
	/**
	 * Recursive method used to get the children of each of the tree element
	 */
	public static void readChildrenData(TreeItem item, ActionFlag operation) {
		if (operation == ActionFlag.COLUMN_ALIGNMENT)
			readRow(item, operation);
		else if (operation == ActionFlag.READ_ROW)
			_tableText += readRow(item, operation);
	
		TreeItem[] child = item.getItems();
		int parentLevel = _treeLevel;
		
		if (child.length > 0) {
			if (child[0].getData() != null && item.getExpanded()) {
				//since we are moving a level deeper into the tree, we increment _treelevel by 1
				_treeLevel = _treeLevel + 1;
			}
		} else {
			//We have no more children. Since the recursive function is going to go back to its parent tree item, we decrement _treelevel by 1
			_treeLevel = _treeLevel - 1;
		}
		
		for(int i=0; i<child.length; i++) {
			if (child[i].getData() != null && item.getExpanded()) {
				readChildrenData(child[i], operation);
				if (_treeLevel==parentLevel) {
					// am I the last child? if yes, then decrement _treeLevel : if no, then increment _treeLevel
					_treeLevel = ((i==child.length-1) ? (_treeLevel-1) : _treeLevel+1);
				}
			}
		}
	}
	
	/**
	 * Reads the data contained in a specific row in a String array
	 * 
	 * If parameter 'operation' is set to ActionFlag.READ_ROW, this method finds the length of the longest string in each column
	 * If parameter 'operation' is set to ActionFlag.COLUMN_ALIGNMENT, this method reads and returns the row's data.
	 */
	public static String readRow(TreeItem item, ActionFlag operation) {
		String str = "";
		String line = "";
		
		for (int i=0; i<_numCols; i++) {
			if (i==0) {
				for (int j=0; j<_treeLevel; j++) {
					str += "    ";
					if (j==_treeLevel-1)	
						str += "> ";
				}
				str += item.getText(i);
			} else {
				str = item.getText(i);
			}
			
			if (operation == ActionFlag.COLUMN_ALIGNMENT) {
				_maxValueLen[i] = (str.length() > _maxValueLen[i]) ? str.length() : _maxValueLen[i];
			} else if (operation == ActionFlag.READ_ROW) {
				line += str;
				line += repeat(" ", _maxValueLen[i] - str.length());
			}
		}
		
		line += "\n";
		return line;
	}
	
	/**
	 * Repeats a string by the specified number of times
	 */
	public static String repeat(String str, int repeat) {
		String text="";
		while (repeat-- != 0) {
			text += str;
		}
		return text;
	}
			
	
}
