/**********************************************************************
 Copyright (c) 2012 IBM Corporation and others.
 All rights reserved. This program and the accompanying materials
 are made available under the terms of the Eclipse Public License v1.0
 which accompanies this distribution, and is available at
 http://www.eclipse.org/legal/epl-v10.html
 
 Contributors:
     IBM - initial implementation
 **********************************************************************/
package org.eclipse.tptp.platform.iac.administrator.internal.common;

import java.util.Hashtable;

import org.eclipse.osgi.service.debug.DebugOptions;
import org.eclipse.osgi.service.debug.DebugOptionsListener;
import org.eclipse.osgi.service.debug.DebugTrace;
import org.eclipse.tptp.platform.iac.administrator.AdminPlugin;
import org.osgi.framework.BundleContext;

public class Trace implements DebugOptionsListener {
	
	private static Trace instance = new Trace();
	
	public static String TRACESTRING = "/trace";
	public static String TRACESTRING_traceIAC = TRACESTRING + "/IAC";
	public static String TRACESTRING_traceIAC_directory = TRACESTRING + "/IAC/directory";
	public static String TRACESTRING_traceIAC_format = TRACESTRING + "/IAC/format";
	public static String TRACESTRING_startStop = TRACESTRING + "/startStop";
	
	public static boolean TRACING = false;
	public static boolean TRACE_traceIAC = false;
	public static String TRACE_traceIAC_directory = "%TPTP_AC_CONFIG_HOME%";
	public static String TRACE_traceIAC_format = "CBE";
	public static boolean TRACE_startStop = false;
	
	private DebugTrace traceObject;
	
	private Trace() {
		// Nothing to do; just mark private so no on else can instantiate us.
	}
	
	public void optionsChanged(DebugOptions options) {
		
		if( null == this.traceObject )
			this.traceObject = options.newDebugTrace( AdminPlugin.PLUGIN_ID, this.getClass());
		
		Trace.TRACING = options.getBooleanOption( AdminPlugin.PLUGIN_ID + TRACESTRING, false);
		Trace.TRACE_traceIAC = options.getBooleanOption( AdminPlugin.PLUGIN_ID + TRACESTRING_traceIAC, false );
		Trace.TRACE_traceIAC_directory = options.getOption( AdminPlugin.PLUGIN_ID + TRACESTRING_traceIAC_directory, "%TPTP_AC_CONFIG_HOME%");
		Trace.TRACE_traceIAC_format = options.getOption( AdminPlugin.PLUGIN_ID + TRACESTRING_traceIAC_format, "CBE");
		Trace.TRACE_startStop = options.getBooleanOption( AdminPlugin.PLUGIN_ID + TRACESTRING_startStop, false );
		
	}
	
	public static Trace getInstance() {
		return instance;
	}
	
	public static void initialize( BundleContext context, String pluginId ) {
		
		final Hashtable<String,String> props = new Hashtable<String,String>(1);
		props.put(DebugOptions.LISTENER_SYMBOLICNAME, pluginId);
		context.registerService(DebugOptionsListener.class.getName(), Trace.getInstance(), props);
		
		if( Trace.TRACING )
			Trace.getDebugTrace().trace(Trace.TRACESTRING, pluginId + ": trace has started");

		if( Trace.TRACE_traceIAC ) {
			System.setProperty("org.eclipse.tptp.iac.log.level", "DEBUG");
			System.setProperty("org.eclipse.tptp.iac.log.format", Trace.TRACE_traceIAC_format);
			System.setProperty("org.eclipse.tptp.iac.log.directory", Trace.TRACE_traceIAC_directory);
		}
		

	}
	
	public static DebugTrace getDebugTrace() {
		return getInstance().traceObject;
	}
	
}
