/*******************************************************************************
 * Licensed Material - Property of IBM
 * (C) Copyright IBM Corp. 2015 - All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

#include <string.h>
#include "BCITrace.h"
#include "time.h"


BCITrace& BCITrace::instance()
{
	static BCITrace* trace = 0;
	if (!trace)
		trace = new BCITrace();
	return *trace;
}
//------------------------------------------------------------------------------
BCITrace::BCITrace()
	: _file(0)
{
	const char* filePath = getenv("BCIPROBE_LOG_FILE");
	if (filePath != NULL)
	{
		_file = fopen(filePath, "a");
	}
}
//------------------------------------------------------------------------------
BCITrace::~BCITrace()
{
	if (_file)
		fclose(_file);
	_file = 0;
}
//------------------------------------------------------------------------------
void BCITrace::traceBCI(const char* fn,
			  unsigned line,
			  const char* fmt,
			  va_list va)
{
	trace("BCI", fn, line, fmt, va);
}
//------------------------------------------------------------------------------
void BCITrace::traceJClass(const char* fn,
					       unsigned line,
						   const char* fmt,
						   va_list va)
{
	trace("JClass", fn, line, fmt, va);
}
//------------------------------------------------------------------------------
void BCITrace::trace(const char* groupName,
					 const char* fn,
				     unsigned ln,
					 const char* fmt,
					 va_list va)
{
	if (!_file)
		return;

	static char* line = 0;
	if (!line)
		line = (char*)malloc(128000);

	int pos = 0;
	time_t t = time(0);
	char* stime = ctime(&t);
	char* crtrn = strrchr(stime, '\r');
	char* nln = strrchr(stime, '\n');
	if (crtrn)
		*crtrn = ' ';
	if (nln)
		*nln = ' ';

	pos += sprintf(line + pos, "%s: %s: ", stime, groupName);

	if (fn)
		pos += sprintf(line+pos, " %s", fn);

	if (ln)
		pos += sprintf(line+pos, " %d: ", ln);

	pos += vsprintf(line+pos, fmt, va);
	line[pos++] = '\n';
	line[pos] = '\0';

#if defined(MVS) || defined(__OS400__)
	/* On EBCDIC systems, convert ASCII to EBCDIC */
	char *data = (char*)malloc(strlen(line) + 200);
	strcpy(data, line);
	__atoe(data);
	fwrite(data, 1, pos, _file);
	free(data);
#else
	fwrite(line, 1, pos, _file);
#endif

	fflush(_file);
}
